// Copyright(c) 2015-present, Gabi Melman & spdlog contributors.
// Distributed under the MIT License (http://opensource.org/licenses/MIT)

#pragma once

namespace spdlog {
class logger;
class formatter;

namespace sinks {
class sink;
}

} // namespace spdlog
